'use strict';

let gulp = require('gulp');
let phpmd = require('gulp-phpmd-plugin');
let Elixir = require('laravel-elixir');

let path = require('path');

/*
 |----------------------------------------------------------------
 | PHP Mess Detector
 |----------------------------------------------------------------
 |
 | This task triggers the entire PHP Mess Detector
 | suite and shows notifications indicating the
 |  success or failure that test suite.
 |
 */
// noinspection JSUnresolvedFunction
Elixir.extend('phpmd', function initPHPMD (src, options) {
    // SETUP:       Notifications
    let notify = new Elixir.Notification();

    // SETUP:       Notification for errors
    let onError = function onError (err) {
        // REMOVE:      ANSI code from message
        err.message = err.message.replace(
            /[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, '');

        // NOTIFY
        notify.error(err, 'PHPMD Failed');
        this.emit('end');
    };

    // DEFAULT:       source
    let source = src || ['**/*.php', '!vendor/**/*', '!node_modules'];


    // ADD:         Task
    new Elixir.Task('phpmd', function taskPhpcs () {
        this.recordStep('Run PHP Mess Detector over source files.');

        // DEFAULT:       settings
        let defaults = {
            bin:     './vendor/bin/phpmd',
            format:  'text',
            ruleset: 'phpmd.xml',
        };
        let settings = Object.assign(defaults, options);

        // Normalize path
        settings.bin = path.normalize(settings.bin);

        // GULP:        Run it
        return gulp.src(source)
            // Validate files using PHP Code Sniffer
            .pipe(phpmd(settings))
            // Send through reporter pipes
            .pipe(phpmd.reporter('log'))
            .pipe(phpmd.reporter('fail'))
            // Send errors to elixir
            .on('error', onError);
    }).watch(source);
});

